@@ -13,7 +13,7 @@
        generate:
                sphinx-autogen -o source/generated source/*.rst
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -24,7 +24,7 @@ import pydoc
 import re
 import sys
 import warnings
-from typing import Any, Callable, Dict, List, Set, Tuple
+from typing import Any, Callable, Dict, List, NamedTuple, Set, Tuple
 
 from jinja2 import BaseLoader, FileSystemLoader, TemplateNotFound
 from jinja2.sandbox import SandboxedEnvironment
@@ -62,22 +62,29 @@ class DummyApplication:
         self._warncount = 0
         self.warningiserror = False
 
-    def emit_firstresult(self, *args) -> None:
+    def emit_firstresult(self, *args: Any) -> None:
         pass
 
 
+AutosummaryEntry = NamedTuple('AutosummaryEntry', [('name', str),
+                                                   ('path', str),
+                                                   ('template', str)])
+
+
 def setup_documenters(app: Any) -> None:
     from sphinx.ext.autodoc import (
         ModuleDocumenter, ClassDocumenter, ExceptionDocumenter, DataDocumenter,
         FunctionDocumenter, MethodDocumenter, AttributeDocumenter,
         InstanceAttributeDocumenter, DecoratorDocumenter, PropertyDocumenter,
-        SlotsAttributeDocumenter,
+        SlotsAttributeDocumenter, DataDeclarationDocumenter,
+        SingledispatchFunctionDocumenter,
     )
     documenters = [
         ModuleDocumenter, ClassDocumenter, ExceptionDocumenter, DataDocumenter,
         FunctionDocumenter, MethodDocumenter, AttributeDocumenter,
         InstanceAttributeDocumenter, DecoratorDocumenter, PropertyDocumenter,
-        SlotsAttributeDocumenter,
+        SlotsAttributeDocumenter, DataDeclarationDocumenter,
+        SingledispatchFunctionDocumenter,
     ]  # type: List[Type[Documenter]]
     for documenter in documenters:
         app.registry.add_documenter(documenter.objtype, documenter)
@@ -148,7 +155,7 @@ def generate_autosummary_content(name: str, obj: Any, parent: Any,
             return app.emit_firstresult('autodoc-skip-member', objtype, name,
                                         obj, False, {})
         except Exception as exc:
-            logger.warning(__('autosummary: failed to determine %r to be documented.'
+            logger.warning(__('autosummary: failed to determine %r to be documented, '
                               'the following exception was raised:\n%s'),
                            name, exc, type='autosummary')
             return False
@@ -223,7 +230,8 @@ def generate_autosummary_docs(sources: List[str], output_dir: str = None,
                               suffix: str = '.rst', warn: Callable = None,
                               info: Callable = None, base_path: str = None,
                               builder: Builder = None, template_dir: str = None,
-                              imported_members: bool = False, app: Any = None) -> None:
+                              imported_members: bool = False, app: Any = None,
+                              overwrite: bool = True) -> None:
     if info:
         warnings.warn('info argument for generate_autosummary_docs() is deprecated.',
                       RemovedInSphinx40Warning)
@@ -259,51 +267,58 @@ def generate_autosummary_docs(sources: List[str], output_dir: str = None,
     new_files = []
 
     # write
-    for name, path, template_name in sorted(set(items), key=str):
-        if path is None:
+    for entry in sorted(set(items), key=str):
+        if entry.path is None:
             # The corresponding autosummary:: directive did not have
             # a :toctree: option
             continue
 
-        path = output_dir or os.path.abspath(path)
+        path = output_dir or os.path.abspath(entry.path)
         ensuredir(path)
 
         try:
-            name, obj, parent, mod_name = import_by_name(name)
+            name, obj, parent, mod_name = import_by_name(entry.name)
         except ImportError as e:
-            _warn('[autosummary] failed to import %r: %s' % (name, e))
+            _warn(__('[autosummary] failed to import %r: %s') % (entry.name, e))
             continue
 
-        fn = os.path.join(path, name + suffix)
+        content = generate_autosummary_content(name, obj, parent, template, entry.template,
+                                               imported_members, app)
 
-        # skip it if it exists
-        if os.path.isfile(fn):
-            continue
+        filename = os.path.join(path, name + suffix)
+        if os.path.isfile(filename):
+            with open(filename) as f:
+                old_content = f.read()
 
-        new_files.append(fn)
-
-        with open(fn, 'w') as f:
-            rendered = generate_autosummary_content(name, obj, parent,
-                                                    template, template_name,
-                                                    imported_members, app)
-            f.write(rendered)
+            if content == old_content:
+                continue
+            elif overwrite:  # content has changed
+                with open(filename, 'w') as f:
+                    f.write(content)
+                new_files.append(filename)
+        else:
+            with open(filename, 'w') as f:
+                f.write(content)
+            new_files.append(filename)
 
     # descend recursively to new files
     if new_files:
         generate_autosummary_docs(new_files, output_dir=output_dir,
                                   suffix=suffix, warn=warn, info=info,
                                   base_path=base_path, builder=builder,
-                                  template_dir=template_dir, app=app)
+                                  template_dir=template_dir,
+                                  imported_members=imported_members, app=app,
+                                  overwrite=overwrite)
 
 
 # -- Finding documented entries in files ---------------------------------------
 
-def find_autosummary_in_files(filenames: List[str]) -> List[Tuple[str, str, str]]:
+def find_autosummary_in_files(filenames: List[str]) -> List[AutosummaryEntry]:
     """Find out what items are documented in source/*.rst.
 
     See `find_autosummary_in_lines`.
     """
-    documented = []  # type: List[Tuple[str, str, str]]
+    documented = []  # type: List[AutosummaryEntry]
     for filename in filenames:
         with open(filename, encoding='utf-8', errors='ignore') as f:
             lines = f.read().splitlines()
@@ -312,7 +327,7 @@ def find_autosummary_in_files(filenames: List[str]) -> List[Tuple[str, str, str]
 
 
 def find_autosummary_in_docstring(name: str, module: Any = None, filename: str = None
-                                  ) -> List[Tuple[str, str, str]]:
+                                  ) -> List[AutosummaryEntry]:
     """Find out what items are documented in the given object's docstring.
 
     See `find_autosummary_in_lines`.
@@ -332,7 +347,7 @@ def find_autosummary_in_docstring(name: str, module: Any = None, filename: str =
 
 
 def find_autosummary_in_lines(lines: List[str], module: Any = None, filename: str = None
-                              ) -> List[Tuple[str, str, str]]:
+                              ) -> List[AutosummaryEntry]:
     """Find out what items appear in autosummary:: directives in the
     given lines.
 
@@ -352,7 +367,7 @@ def find_autosummary_in_lines(lines: List[str], module: Any = None, filename: st
     toctree_arg_re = re.compile(r'^\s+:toctree:\s*(.*?)\s*$')
     template_arg_re = re.compile(r'^\s+:template:\s*(.*?)\s*$')
 
-    documented = []  # type: List[Tuple[str, str, str]]
+    documented = []  # type: List[AutosummaryEntry]
 
     toctree = None  # type: str
     template = None
@@ -386,7 +401,7 @@ def find_autosummary_in_lines(lines: List[str], module: Any = None, filename: st
                 if current_module and \
                    not name.startswith(current_module + '.'):
                     name = "%s.%s" % (current_module, name)
-                documented.append((name, toctree, template))
+                documented.append(AutosummaryEntry(name, toctree, template))
                 continue
 
             if not line.strip() or line.startswith(base_indent + " "):
