@@ -12,7 +12,6 @@ import platform
 
 import pytest
 
-from sphinx.ext.autodoc import merge_autodoc_default_flags
 from test_autodoc import do_autodoc
 
 IS_PYPY = platform.python_implementation() == 'PyPy'
@@ -32,49 +31,49 @@ def test_autoclass_content_class(app):
         '   :module: target.autoclass_content',
         '',
         '   A class having no __init__, no __new__',
-        '   ',
+        '',
         '',
         '.. py:class:: B()',
         '   :module: target.autoclass_content',
         '',
         '   A class having __init__(no docstring), no __new__',
-        '   ',
+        '',
         '',
         '.. py:class:: C()',
         '   :module: target.autoclass_content',
         '',
         '   A class having __init__, no __new__',
-        '   ',
+        '',
         '',
         '.. py:class:: D',
         '   :module: target.autoclass_content',
         '',
         '   A class having no __init__, __new__(no docstring)',
-        '   ',
+        '',
         '',
         '.. py:class:: E',
         '   :module: target.autoclass_content',
         '',
         '   A class having no __init__, __new__',
-        '   ',
+        '',
         '',
         '.. py:class:: F()',
         '   :module: target.autoclass_content',
         '',
         '   A class having both __init__ and __new__',
-        '   ',
+        '',
         '',
         '.. py:class:: G()',
         '   :module: target.autoclass_content',
         '',
         '   A class inherits __init__ without docstring.',
-        '   ',
+        '',
         '',
         '.. py:class:: H()',
         '   :module: target.autoclass_content',
         '',
         '   A class inherits __new__ without docstring.',
-        '   '
+        '',
     ]
 
 
@@ -92,49 +91,49 @@ def test_autoclass_content_init(app):
         '   :module: target.autoclass_content',
         '',
         '   A class having no __init__, no __new__',
-        '   ',
+        '',
         '',
         '.. py:class:: B()',
         '   :module: target.autoclass_content',
         '',
         '   A class having __init__(no docstring), no __new__',
-        '   ',
+        '',
         '',
         '.. py:class:: C()',
         '   :module: target.autoclass_content',
         '',
         '   __init__ docstring',
-        '   ',
+        '',
         '',
         '.. py:class:: D',
         '   :module: target.autoclass_content',
         '',
         '   A class having no __init__, __new__(no docstring)',
-        '   ',
+        '',
         '',
         '.. py:class:: E',
         '   :module: target.autoclass_content',
         '',
         '   __new__ docstring',
-        '   ',
+        '',
         '',
         '.. py:class:: F()',
         '   :module: target.autoclass_content',
         '',
         '   __init__ docstring',
-        '   ',
+        '',
         '',
         '.. py:class:: G()',
         '   :module: target.autoclass_content',
         '',
         '   __init__ docstring',
-        '   ',
+        '',
         '',
         '.. py:class:: H()',
         '   :module: target.autoclass_content',
         '',
         '   __new__ docstring',
-        '   '
+        '',
     ]
 
 
@@ -152,59 +151,59 @@ def test_autoclass_content_both(app):
         '   :module: target.autoclass_content',
         '',
         '   A class having no __init__, no __new__',
-        '   ',
+        '',
         '',
         '.. py:class:: B()',
         '   :module: target.autoclass_content',
         '',
         '   A class having __init__(no docstring), no __new__',
-        '   ',
+        '',
         '',
         '.. py:class:: C()',
         '   :module: target.autoclass_content',
         '',
         '   A class having __init__, no __new__',
-        '   ',
+        '',
         '   __init__ docstring',
-        '   ',
+        '',
         '',
         '.. py:class:: D',
         '   :module: target.autoclass_content',
         '',
         '   A class having no __init__, __new__(no docstring)',
-        '   ',
+        '',
         '',
         '.. py:class:: E',
         '   :module: target.autoclass_content',
         '',
         '   A class having no __init__, __new__',
-        '   ',
+        '',
         '   __new__ docstring',
-        '   ',
+        '',
         '',
         '.. py:class:: F()',
         '   :module: target.autoclass_content',
         '',
         '   A class having both __init__ and __new__',
-        '   ',
+        '',
         '   __init__ docstring',
-        '   ',
+        '',
         '',
         '.. py:class:: G()',
         '   :module: target.autoclass_content',
         '',
         '   A class inherits __init__ without docstring.',
-        '   ',
+        '',
         '   __init__ docstring',
-        '   ',
+        '',
         '',
         '.. py:class:: H()',
         '   :module: target.autoclass_content',
         '',
         '   A class inherits __new__ without docstring.',
-        '   ',
+        '',
         '   __new__ docstring',
-        '   '
+        '',
     ]
 
 
@@ -218,7 +217,7 @@ def test_autodoc_inherit_docstrings(app):
         '   :module: target.inheritance',
         '',
         '   Inherited function.',
-        '   '
+        '',
     ]
 
     # disable autodoc_inherit_docstrings
@@ -241,38 +240,38 @@ def test_autodoc_docstring_signature(app):
         '.. py:class:: DocstringSig',
         '   :module: target',
         '',
-        '   ',
+        '',
         '   .. py:method:: DocstringSig.meth(FOO, BAR=1) -> BAZ',
         '      :module: target',
-        '   ',
+        '',
         '      First line of docstring',
-        '      ',
+        '',
         '      rest of docstring',
-        '      ',
-        '   ',
+        '',
+        '',
         '   .. py:method:: DocstringSig.meth2()',
         '      :module: target',
-        '   ',
+        '',
         '      First line, no signature',
         '      Second line followed by indentation::',
-        '      ',
+        '',
         '          indented line',
-        '      ',
-        '   ',
+        '',
+        '',
         '   .. py:method:: DocstringSig.prop1',
         '      :module: target',
         '      :property:',
-        '   ',
+        '',
         '      First line of docstring',
-        '      ',
-        '   ',
+        '',
+        '',
         '   .. py:method:: DocstringSig.prop2',
         '      :module: target',
         '      :property:',
-        '   ',
+        '',
         '      First line of docstring',
         '      Second line of docstring',
-        '      '
+        '',
     ]
 
     # disable autodoc_docstring_signature
@@ -283,41 +282,41 @@ def test_autodoc_docstring_signature(app):
         '.. py:class:: DocstringSig',
         '   :module: target',
         '',
-        '   ',
+        '',
         '   .. py:method:: DocstringSig.meth()',
         '      :module: target',
-        '   ',
+        '',
         '      meth(FOO, BAR=1) -> BAZ',
         '      First line of docstring',
-        '      ',
+        '',
         '              rest of docstring',
-        '              ',
-        '      ',
-        '   ',
+        '',
+        '',
+        '',
         '   .. py:method:: DocstringSig.meth2()',
         '      :module: target',
-        '   ',
+        '',
         '      First line, no signature',
         '      Second line followed by indentation::',
-        '      ',
+        '',
         '          indented line',
-        '      ',
-        '   ',
+        '',
+        '',
         '   .. py:method:: DocstringSig.prop1',
         '      :module: target',
         '      :property:',
-        '   ',
+        '',
         '      DocstringSig.prop1(self)',
         '      First line of docstring',
-        '      ',
-        '   ',
+        '',
+        '',
         '   .. py:method:: DocstringSig.prop2',
         '      :module: target',
         '      :property:',
-        '   ',
+        '',
         '      First line of docstring',
         '      Second line of docstring',
-        '      '
+        '',
     ]
 
 
@@ -398,13 +397,13 @@ def test_autoclass_content_and_docstring_signature_both(app):
         '   :module: target.docstring_signature',
         '',
         '   B(foo, bar, baz)',
-        '   ',
+        '',
         '',
         '.. py:class:: C(foo, bar)',
         '   :module: target.docstring_signature',
         '',
         '   C(foo, bar, baz)',
-        '   ',
+        '',
         '',
         '.. py:class:: D(foo, bar, baz)',
         '   :module: target.docstring_signature',
@@ -440,33 +439,32 @@ def test_mocked_module_imports(app, warning):
         '   :module: target.need_mocks',
         '',
         '   TestAutodoc docstring.',
-        '   ',
-        '   ',
+        '',
+        '',
         '   .. py:method:: TestAutodoc.decoratedMethod()',
         '      :module: target.need_mocks',
-        '   ',
+        '',
         '      TestAutodoc::decoratedMethod docstring',
-        '      ',
+        '',
         '',
         '.. py:function:: decoratedFunction()',
         '   :module: target.need_mocks',
         '',
         '   decoratedFunction docstring',
-        '   ',
+        '',
         '',
         '.. py:function:: func(arg: missing_module.Class)',
         '   :module: target.need_mocks',
         '',
         '   a function takes mocked object as an argument',
-        '   '
+        '',
     ]
     assert warning.getvalue() == ''
 
 
-@pytest.mark.sphinx('html', testroot='ext-autodoc')
+@pytest.mark.sphinx('html', testroot='ext-autodoc',
+                    confoverrides={'autodoc_typehints': "signature"})
 def test_autodoc_typehints_signature(app):
-    app.config.autodoc_typehints = "signature"
-
     options = {"members": None,
                "undoc-members": True}
     actual = do_autodoc(app, 'module', 'target.typehints', options)
@@ -478,14 +476,22 @@ def test_autodoc_typehints_signature(app):
         '.. py:class:: Math(s: str, o: object = None)',
         '   :module: target.typehints',
         '',
-        '   ',
+        '',
         '   .. py:method:: Math.decr(a: int, b: int = 1) -> int',
         '      :module: target.typehints',
-        '   ',
-        '   ',
+        '',
+        '',
+        '   .. py:method:: Math.horse(a: str, b: int) -> None',
+        '      :module: target.typehints',
+        '',
+        '',
         '   .. py:method:: Math.incr(a: int, b: int = 1) -> int',
         '      :module: target.typehints',
-        '   ',
+        '',
+        '',
+        '   .. py:method:: Math.nothing() -> None',
+        '      :module: target.typehints',
+        '',
         '',
         '.. py:function:: complex_func(arg1: str, arg2: List[int], arg3: Tuple[int, '
         'Union[str, Unknown]] = None, *args: str, **kwargs: str) -> None',
@@ -498,14 +504,17 @@ def test_autodoc_typehints_signature(app):
         '',
         '.. py:function:: incr(a: int, b: int = 1) -> int',
         '   :module: target.typehints',
+        '',
+        '',
+        '.. py:function:: missing_attr(c, a: str, b: Optional[str] = None) -> str',
+        '   :module: target.typehints',
         ''
     ]
 
 
-@pytest.mark.sphinx('html', testroot='ext-autodoc')
+@pytest.mark.sphinx('html', testroot='ext-autodoc',
+                    confoverrides={'autodoc_typehints': "none"})
 def test_autodoc_typehints_none(app):
-    app.config.autodoc_typehints = "none"
-
     options = {"members": None,
                "undoc-members": True}
     actual = do_autodoc(app, 'module', 'target.typehints', options)
@@ -517,14 +526,22 @@ def test_autodoc_typehints_none(app):
         '.. py:class:: Math(s, o=None)',
         '   :module: target.typehints',
         '',
-        '   ',
+        '',
         '   .. py:method:: Math.decr(a, b=1)',
         '      :module: target.typehints',
-        '   ',
-        '   ',
+        '',
+        '',
+        '   .. py:method:: Math.horse(a, b)',
+        '      :module: target.typehints',
+        '',
+        '',
         '   .. py:method:: Math.incr(a, b=1)',
         '      :module: target.typehints',
-        '   ',
+        '',
+        '',
+        '   .. py:method:: Math.nothing()',
+        '      :module: target.typehints',
+        '',
         '',
         '.. py:function:: complex_func(arg1, arg2, arg3=None, *args, **kwargs)',
         '   :module: target.typehints',
@@ -536,16 +553,19 @@ def test_autodoc_typehints_none(app):
         '',
         '.. py:function:: incr(a, b=1)',
         '   :module: target.typehints',
+        '',
+        '',
+        '.. py:function:: missing_attr(c, a, b=None)',
+        '   :module: target.typehints',
         ''
     ]
 
 
 @pytest.mark.sphinx('text', testroot='ext-autodoc',
-                    confoverrides={'extensions': ['sphinx.ext.autodoc.typehints'],
-                                   'autodoc_typehints': 'description'})
+                    confoverrides={'autodoc_typehints': "description"})
 def test_autodoc_typehints_description(app):
     app.build()
-    context = (app.outdir / 'index.txt').text()
+    context = (app.outdir / 'index.txt').read_text()
     assert ('target.typehints.incr(a, b=1)\n'
             '\n'
             '   Parameters:\n'
@@ -558,27 +578,6 @@ def test_autodoc_typehints_description(app):
             in context)
 
 
-@pytest.mark.sphinx('html', testroot='ext-autodoc')
-@pytest.mark.filterwarnings('ignore:autodoc_default_flags is now deprecated.')
-def test_merge_autodoc_default_flags1(app):
-    app.config.autodoc_default_flags = ['members', 'undoc-members']
-    merge_autodoc_default_flags(app, app.config)
-    assert app.config.autodoc_default_options == {'members': None,
-                                                  'undoc-members': None}
-
-
-@pytest.mark.sphinx('html', testroot='ext-autodoc')
-@pytest.mark.filterwarnings('ignore:autodoc_default_flags is now deprecated.')
-def test_merge_autodoc_default_flags2(app):
-    app.config.autodoc_default_flags = ['members', 'undoc-members']
-    app.config.autodoc_default_options = {'members': 'this,that,order',
-                                          'inherited-members': 'this'}
-    merge_autodoc_default_flags(app, app.config)
-    assert app.config.autodoc_default_options == {'members': None,
-                                                  'undoc-members': None,
-                                                  'inherited-members': 'this'}
-
-
 @pytest.mark.sphinx('html', testroot='ext-autodoc')
 def test_autodoc_default_options(app):
     # no settings
@@ -588,7 +587,7 @@ def test_autodoc_default_options(app):
     actual = do_autodoc(app, 'class', 'target.CustomIter')
     assert '   .. py:method:: target.CustomIter' not in actual
     actual = do_autodoc(app, 'module', 'target')
-    assert '.. py:function:: save_traceback(app: Sphinx) -> str' not in actual
+    assert '.. py:function:: save_traceback(app)' not in actual
 
     # with :members:
     app.config.autodoc_default_options = {'members': None}
@@ -652,16 +651,6 @@ def test_autodoc_default_options(app):
     assert '   .. py:method:: CustomIter.snafucate()' in actual
     assert '      Makes this snafucated.' in actual
 
-    # with :imported-members:
-    app.config.autodoc_default_options = {
-        'members': None,
-        'imported-members': None,
-        'ignore-module-all': None,
-    }
-    actual = do_autodoc(app, 'module', 'target')
-    print('\n'.join(actual))
-    assert '.. py:function:: save_traceback(app: Sphinx) -> str' in actual
-
 
 @pytest.mark.sphinx('html', testroot='ext-autodoc')
 def test_autodoc_default_options_with_values(app):
