@@ -4,7 +4,7 @@
 
     Test various Sphinx-specific markup extensions.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -230,6 +230,13 @@ def get_verifier(verify, verify_re):
         '<p><span class="guilabel">Foo</span></p>',
         r'\sphinxguilabel{Foo}',
     ),
+    (
+        # kbd role
+        'verify',
+        ':kbd:`space`',
+        '<p><kbd class="kbd docutils literal notranslate">space</kbd></p>',
+        '\\sphinxkeyboard{\\sphinxupquote{space}}',
+    ),
     (
         # non-interpolation of dashes in option role
         'verify_re',
