@@ -2,7 +2,7 @@
     test_transforms_post_transforms_code
     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -13,7 +13,7 @@ import pytest
 def test_trim_doctest_flags_html(app, status, warning):
     app.build()
 
-    result = (app.outdir / 'index.html').text(encoding='utf8')
+    result = (app.outdir / 'index.html').read_text()
     assert 'FOO' not in result
     assert 'BAR' in result
     assert 'BAZ' not in result
@@ -25,7 +25,7 @@ def test_trim_doctest_flags_html(app, status, warning):
 def test_trim_doctest_flags_latex(app, status, warning):
     app.build()
 
-    result = (app.outdir / 'python.tex').text(encoding='utf8')
+    result = (app.outdir / 'python.tex').read_text()
     assert 'FOO' not in result
     assert 'BAR' in result
     assert 'BAZ' not in result
